@@ -3,11 +3,12 @@
 #' @description define an object in an R session as a data greta array for use
 #'   as data in a greta model.
 #' @param x an R object that can be coerced to a greta_array (see details).
-#' @details \code{as_data()} can currently convert R objects to greta_arrays if they are
-#'   numeric or logical vectors, matrices or arrays; or if they are dataframes
-#'   with only numeric (including integer) or logical elements. Logical elements
-#'   are always converted to numerics. R objects cannot be converted if they
-#'   contain missing (\code{NA}) or infinite (\code{-Inf} or \code{Inf}) values.
+#' @details \code{as_data()} can currently convert R objects to greta_arrays if
+#'   they are numeric or logical vectors, matrices or arrays; or if they are
+#'   dataframes with only numeric (including integer) or logical elements.
+#'   Logical elements are always converted to numerics. R objects cannot be
+#'   converted if they contain missing (\code{NA}) or infinite (\code{-Inf} or
+#'   \code{Inf}) values.
 #' @export
 #' @examples
 #' \dontrun{
@@ -17,7 +18,8 @@
 #'
 #' vec <- rnorm(10)
 #' mat <- matrix(seq_len(3 * 4), nrow = 3)
-#' arr <- array(sample(c(TRUE, FALSE), 2 * 2 * 2, replace = TRUE), dim = c(2, 2, 2))
+#' arr <- array(sample(c(TRUE, FALSE), 2 * 2 * 2, replace = TRUE),
+#'              dim = c(2, 2, 2))
 #' (a <- as_data(vec))
 #' (b <- as_data(mat))
 #' (c <- as_data(arr))
@@ -29,20 +31,23 @@
 #'                  x3 = sample(c(TRUE, FALSE), 10, replace = TRUE))
 #' (d <- as_data(df))
 #' }
-as_data <- function (x)
-  UseMethod('as_data', x)
+as_data <- function(x) {
+  check_tf_version("error")
+  UseMethod("as_data", x)
+}
 
 # if it's already a *data* greta_array fine, else error
 #' @export
-as_data.greta_array <- function (x) {
-  if (!inherits(x$node, 'data_node')) {
-    stop ('cannot coerce a non-data greta_array to data',
-          call. = FALSE)
+as_data.greta_array <- function(x) {
+  if (!inherits(get_node(x), "data_node")) {
+    stop("cannot coerce a non-data greta_array to data",
+         call. = FALSE)
   }
   x
 }
 
 # otherwise try to coerce to a greta array
 #' @export
-as_data.default <- function (x)
+as_data.default <- function(x) {
   as.greta_array(x)
+}
