@@ -1,33 +1,55 @@
 data_node <- R6Class(
-  'data_node',
+  "data_node",
   inherit = node,
   public = list(
 
-    initialize = function (data) {
+    initialize = function(data) {
 
-      # coerce data from common formats to an array here
-      data <- as.array(data)
-
-      # coerce 1D arrays to column vectors
-      if (length(dim(data)) == 1)
-        dim(data) <- c(dim(data), 1)
+      # coerce to an array with 2+ dimensions
+      data <- as_2D_array(data)
 
       # update and store array and store dimension
       super$initialize(dim = dim(data), value = data)
 
     },
 
-    tf = function (dag) {
-      assign(dag$tf_name(self),
-             tf$constant(self$value(), dtype = tf_float()),
-             envir = dag$tf_environment)
+    tf = function(dag) {
+
+      tfe <- dag$tf_environment
+      tf_name <- dag$tf_name(self)
+
+      value <- self$value()
+      shape <- to_shape(c(1, dim(value)))
+      value <- add_first_dim(value)
+
+      # under some circumstances we define data as constants, but normally as
+      # placeholders
+      using_constants <- !is.null(greta_stash$data_as_constants)
+
+      if (using_constants) {
+
+        tensor <- tf$constant(value = value,
+                              dtype = tf_float(),
+                              shape = shape)
+
+      } else {
+
+        tensor <- tf$placeholder(shape = shape,
+                                 dtype = tf_float())
+        tfe$data_list[[tf_name]] <- value
+
+      }
+
+      # create placeholder
+      assign(tf_name, tensor, envir = tfe)
+
     }
   )
 )
 
 # a node for applying operations to values
 operation_node <- R6Class(
-  'operation_node',
+  "operation_node",
   inherit = node,
   public = list(
 
@@ -35,44 +57,53 @@ operation_node <- R6Class(
     operation = NA,
     operation_args = NA,
     arguments = list(),
+    tf_function_env = NA,
 
-    initialize = function (operation,
-                           ...,
-                           dimfun = NULL,
-                           operation_args = list(),
-                           tf_operation = NULL,
-                           value = NULL) {
+    # named greta arrays giving different representations of the greta array
+    # represented by this node that have already been calculated, to be used for
+    # computational speedups or numerical stability. E.g. a logarithm or a
+    # cholesky factor
+    representations = list(),
+
+    initialize = function(operation,
+                          ...,
+                          dim = NULL,
+                          operation_args = list(),
+                          tf_operation = NULL,
+                          value = NULL,
+                          representations = list(),
+                          tf_function_env = parent.frame(3)) {
 
       # coerce all arguments to nodes, and remember the operation
       dots <- lapply(list(...), as.greta_array)
       for (greta_array in dots)
-        self$add_argument(greta_array$node)
+        self$add_argument(get_node(greta_array))
 
       self$operation_name <- operation
       self$operation <- tf_operation
       self$operation_args <- operation_args
+      self$representations <- representations
+      self$tf_function_env <- tf_function_env
 
-      # work out the dimensions of the new greta array, if NULL assume an elementwise
-      # operation and get the largest number of each dimension, otherwise expect
-      # a function to be passed which will calculate it from the provided list
-      # of nodes arguments
-      if (is.null(dimfun))
+      # work out the dimensions of the new greta array, if NULL assume an
+      # elementwise operation and get the largest number of each dimension,
+      # otherwise expect a function to be passed which will calculate it from
+      # the provided list of nodes arguments
+      if (is.null(dim))
         dim <- do.call(pmax, lapply(dots, dim))
-      else
-        dim <- dimfun(dots)
 
       # assign empty value of the right dimension, or the values passed via the
       # operation
       if (is.null(value))
         value <- unknowns(dim = dim)
       else if (!all.equal(dim(value), dim))
-        stop ('values have the wrong dimension so cannot be used')
+        stop("values have the wrong dimension so cannot be used")
 
       super$initialize(dim, value)
 
     },
 
-    add_argument = function (argument) {
+    add_argument = function(argument) {
 
       # guess at a name, coerce to a node, and add as a child
       parameter <- to_node(argument)
@@ -80,7 +111,7 @@ operation_node <- R6Class(
 
     },
 
-    tf = function (dag) {
+    tf = function(dag) {
 
       # fetch the tensors for the environment
       arg_tf_names <- lapply(self$children, dag$tf_name)
@@ -90,8 +121,15 @@ operation_node <- R6Class(
       if (length(self$operation_args) > 0)
         args <- c(args, self$operation_args)
 
-      # apply function on tensors
-      node <- do.call(self$operation, args)
+      # if there are multiple args, reconcile batch dimensions now
+      args <- match_batches(args)
+
+      # get the tensorflow function
+      operation <- eval(parse(text = self$operation),
+                        envir = self$tf_function_env)
+
+      # apply the function on tensors
+      node <- do.call(operation, args)
 
       # assign it in the environment
       assign(dag$tf_name(self), node, envir = dag$tf_environment)
@@ -100,8 +138,8 @@ operation_node <- R6Class(
   )
 )
 
-variable_node <- R6Class (
-  'variable_node',
+variable_node <- R6Class(
+  "variable_node",
   inherit = node,
   public = list(
 
@@ -109,15 +147,15 @@ variable_node <- R6Class (
     lower = -Inf,
     upper = Inf,
 
-    initialize = function (lower = -Inf, upper = Inf, dim = 1) {
+    initialize = function(lower = -Inf, upper = Inf, dim = 1) {
 
       good_types <- is.numeric(lower) && length(lower) == 1 &
         is.numeric(upper) && length(upper) == 1
 
       if (!good_types) {
 
-        stop ('lower and upper must be numeric vectors of length 1',
-              call. = FALSE)
+        stop("lower and upper must be numeric vectors of length 1",
+             call. = FALSE)
 
       }
 
@@ -127,38 +165,38 @@ variable_node <- R6Class (
       # find constraint type
       if (lower == -Inf & upper == Inf) {
 
-        self$constraint <- 'none'
+        self$constraint <- "none"
         bad_limits <- FALSE
 
       } else if (lower == -Inf & upper != Inf) {
 
-        self$constraint <- 'low'
+        self$constraint <- "low"
         bad_limits <- !is.finite(upper)
 
       } else if (lower != -Inf & upper == Inf) {
 
-        self$constraint <- 'high'
+        self$constraint <- "high"
         bad_limits <- !is.finite(lower)
 
       } else if (lower != -Inf & upper != Inf) {
 
-        self$constraint <- 'both'
+        self$constraint <- "both"
         bad_limits <- !is.finite(lower) | !is.finite(upper)
 
       }
 
       if (bad_limits) {
 
-        stop ('lower and upper must either be -Inf (lower only), ',
-              'Inf (upper only) or finite scalars',
-              call. = FALSE)
+        stop("lower and upper must either be -Inf (lower only), ",
+             "Inf (upper only) or finite scalars",
+             call. = FALSE)
 
       }
 
       if (lower >= upper) {
 
-        stop ('upper bound must be greater than lower bound',
-              call. = FALSE)
+        stop("upper bound must be greater than lower bound",
+             call. = FALSE)
 
       }
 
@@ -169,22 +207,15 @@ variable_node <- R6Class (
 
     },
 
-    tf = function (dag) {
+    tf = function(dag) {
 
-      # make a Variable tensor to hold the free state
-      tf_obj <- tf$Variable(initial_value = self$value(),
-                            dtype = tf_float())
-
-      # assign this as the free state
+      # get the names of the variable and (already-defined) free state version
       tf_name <- dag$tf_name(self)
-      free_name <- sprintf('%s_free', tf_name)
-      assign(free_name,
-             tf_obj,
-             envir = dag$tf_environment)
+      free_name <- sprintf("%s_free", tf_name)
 
-      # get the log jacobian adjustment for the free state
+      # create the log jacobian adjustment for the free state
       tf_adj <- self$tf_adjustment(dag)
-      adj_name <- sprintf('%s_adj', tf_name)
+      adj_name <- sprintf("%s_adj", tf_name)
       assign(adj_name,
              tf_adj,
              envir = dag$tf_environment)
@@ -192,30 +223,36 @@ variable_node <- R6Class (
       # map from the free to constrained state in a new tensor
       tf_free <- get(free_name, envir = dag$tf_environment)
       node <- self$tf_from_free(tf_free, dag$tf_environment)
+
+      # reshape the tensor to the match the variable
+      node <- tf$reshape(node,
+                         shape = to_shape(c(-1, dim(self))))
+
+      # assign as constrained variable
       assign(tf_name,
              node,
              envir = dag$tf_environment)
 
     },
 
-    tf_from_free = function (x, env) {
+    tf_from_free = function(x, env) {
 
       upper <- self$upper
       lower <- self$lower
 
-      if (self$constraint == 'none') {
+      if (self$constraint == "none") {
 
         y <- x
 
-      } else if (self$constraint == 'both') {
+      } else if (self$constraint == "both") {
 
         y <- tf$nn$sigmoid(x) * fl(upper - lower) + fl(lower)
 
-      } else if (self$constraint == 'low') {
+      } else if (self$constraint == "low") {
 
         y <- fl(upper) - tf$exp(x)
 
-      } else if (self$constraint == 'high') {
+      } else if (self$constraint == "high") {
 
         y <- tf$exp(x) + fl(lower)
 
@@ -230,64 +267,75 @@ variable_node <- R6Class (
     },
 
     # adjustments for univariate variables
-    tf_log_jacobian_adjustment = function (free) {
+    tf_log_jacobian_adjustment = function(free) {
 
-      ljac_none <- function (x)
-        fl(0)
+      ljac_none <- function(x) {
+        zero <- tf$constant(0, dtype = tf_float(), shape = shape(1))
+        tf$tile(zero, multiples = list(tf$shape(x)[[0]]))
+      }
 
-      ljac_exp <- function (x)
-        tf$reduce_sum(x)
+      ljac_exp <- function(x) {
+        tf_sum(x, drop = TRUE)
+      }
 
-      ljac_logistic <- function (x) {
+      ljac_logistic <- function(x) {
         lrange <- log(self$upper - self$lower)
-        tf$reduce_sum(x - fl(2) * tf$nn$softplus(x) + lrange)
+        tf_sum(x - fl(2) * tf$nn$softplus(x) + lrange,
+               drop = TRUE)
       }
 
-      ljac_corr_mat <- function (x) {
+      ljac_corr_mat <- function(x) {
 
         # find dimension
-        n <- x$get_shape()$as_list()[1]
-        K <- (1 + sqrt(8 * n + 1)) / 2
+        K <- dim(x)[[2]]
+        n <- (1 + sqrt(8 * K + 1)) / 2
 
-        # draw the rest of the owl
-        l1mz2 <- tf$log(fl(1) - tf$square(tf$tanh(x)))
-        i <- rep(1:(K - 1), (K - 1):1)
-        powers <- tf$constant(K - i - 1,
-                              dtype = tf_float(),
-                              shape = shape(length(i), 1))
-        fl(0.5) * tf$reduce_sum(powers * l1mz2) + tf$reduce_sum(l1mz2)
+        # convert to correlation-scale (-1, 1) & get log jacobian
+        z <- tf$tanh(x)
+
+        free_to_correl_lp <- tf_sum(log(fl(1) - tf$square(z)))
+        free_to_correl_lp <- tf$squeeze(free_to_correl_lp, 1L)
+
+        # split z up into rows
+        z_rows <- tf$split(z, 1:(n - 1), axis = 1L)
+
+        # accumulate log prob within each row
+        lps <- lapply(z_rows, tf_corrmat_row, which = "ljac")
+        correl_to_mat_lp <- tf$add_n(lps)
+
+        free_to_correl_lp + correl_to_mat_lp
 
       }
 
-      ljac_cov_mat <- function (x) {
+      ljac_cov_mat <- function(x) {
 
         # find dimension
-        n <- x$get_shape()$as_list()[1]
+        n <- dim(x)[[2]]
         K <- (sqrt(8 * n + 1) - 1) / 2
 
         k <- seq_len(K)
-        powers <- tf$constant(K - k + 2, dtype = tf_float(), shape = c(K, 1))
-        fl(K * log(2)) + tf$reduce_sum(powers * x[k - 1])
+        powers <- tf$constant(K - k + 2, dtype = tf_float(), shape = shape(K))
+        fl(K * log(2)) + tf_sum(powers * x[, k - 1], drop = TRUE)
 
       }
 
-      fun <- switch (self$constraint,
-                     none = ljac_none,
-                     high = ljac_exp,
-                     low = ljac_exp,
-                     both = ljac_logistic,
-                     correlation_matrix = ljac_corr_mat,
-                     covariance_matrix = ljac_cov_mat)
+      fun <- switch(self$constraint,
+                    none = ljac_none,
+                    high = ljac_exp,
+                    low = ljac_exp,
+                    both = ljac_logistic,
+                    correlation_matrix = ljac_corr_mat,
+                    covariance_matrix = ljac_cov_mat)
 
       fun(free)
 
     },
 
     # create a tensor giving the log jacobian adjustment for this variable
-    tf_adjustment = function (dag) {
+    tf_adjustment = function(dag) {
 
       # find free version of node
-      free_tensor_name <- paste0(dag$tf_name(self), '_free')
+      free_tensor_name <- paste0(dag$tf_name(self), "_free")
       free_tensor <- get(free_tensor_name, envir = dag$tf_environment)
 
       # apply jacobian adjustment to it
@@ -298,11 +346,11 @@ variable_node <- R6Class (
   )
 )
 
-distribution_node <- R6Class (
-  'distribution_node',
+distribution_node <- R6Class(
+  "distribution_node",
   inherit = node,
   public = list(
-    distribution_name = 'no distribution',
+    distribution_name = "no distribution",
     discrete = NA,
     target = NULL,
     user_node = NULL,
@@ -310,18 +358,22 @@ distribution_node <- R6Class (
     truncation = NULL,
     parameters = list(),
 
-    initialize = function (name = 'no distribution', dim = NULL, truncation = NULL, discrete = FALSE) {
+    initialize = function(name = "no distribution",
+                          dim = NULL,
+                          truncation = NULL,
+                          discrete = FALSE) {
 
       super$initialize(dim)
 
-      # for all distributions, set name, store dims and set whether discrete
+      # for all distributions, set name, store dims, and set whether discrete
       self$distribution_name <- name
       self$discrete <- discrete
 
       # initialize the target values of this distribution
       self$add_target(self$create_target(truncation))
 
-      # if there's a truncation, it's different from the bounds, and it's a truncatable distribution, set the truncation
+      # if there's a truncation, it's different from the bounds, and it's a
+      # truncatable distribution, set the truncation
       if (!is.null(truncation) &
           !identical(truncation, self$bounds) &
           !is.null(self$tf_cdf_function)) {
@@ -336,12 +388,12 @@ distribution_node <- R6Class (
     },
 
     # create a target variable node (unconstrained by default)
-    create_target = function (truncation) {
+    create_target = function(truncation) {
       vble(truncation, dim = self$dim)
     },
 
     # create target node, add as a child, and give it this distribution
-    add_target = function (new_target) {
+    add_target = function(new_target) {
 
       # add as x and as a child
       self$target <- new_target
@@ -353,57 +405,64 @@ distribution_node <- R6Class (
       # give self to x as its distribution
       self$target$set_distribution(self)
 
+      # optionally reset any distribution flags relating to the previous target
+      self$reset_target_flags()
+
+    },
+
+    # optional function to reset the flags for target representations whenever a
+    # target is changed
+    reset_target_flags = function() {
+
     },
 
     # replace the existing target node with a new one
-    replace_target = function (new_target) {
+    remove_target = function() {
 
       # remove x from children
       self$remove_child(self$target)
-
-      # add the new one in
-      self$add_target(new_target)
+      self$target <- NULL
 
     },
 
-    tf = function (dag) {
+    tf = function(dag) {
 
-      # define a tensor with this node's log density in env
+      # for distributions, tf assigns a *function* to execute the density
+      density <- function(tf_target) {
 
-      # run the TF version of the density function
-      tf_obj <- self$tf_log_density(dag)
+        # fetch inputs
+        tf_parameters <- self$tf_fetch_parameters(dag)
 
-      # assign the result back to env
+        # ensure x and the parameters are all expanded to have the n_chains
+        # batch dimension, if any have it
+        target_params <- match_batches(c(list(tf_target), tf_parameters))
+        tf_target <- target_params[[1]]
+        tf_parameters <- target_params[-1]
+
+        # calculate log density
+        ld <- self$tf_log_density_function(tf_target, tf_parameters, dag)
+
+        # check for truncation
+        if (!is.null(self$truncation))
+          ld <- ld - self$tf_log_density_offset(tf_parameters)
+
+        ld
+
+      }
+
+      # assign the function to the environment
       assign(dag$tf_name(self),
-             tf_obj,
+             density,
              envir = dag$tf_environment)
 
     },
 
-    # which node to use af the *tf* target (overwritten by some distributions)
-    get_tf_target_node = function () {
+    # which node to use as the *tf* target (overwritten by some distributions)
+    get_tf_target_node = function() {
       self$target
     },
 
-    tf_log_density = function (dag) {
-
-      # fetch inputs
-      tf_target_node <- self$get_tf_target_node()
-      tf_target <- get(dag$tf_name(tf_target_node),
-                       envir = dag$tf_environment)
-      tf_parameters <- self$tf_fetch_parameters(dag)
-
-      # calculate log density
-      ld <- self$tf_log_density_function(tf_target, tf_parameters)
-
-      # check for truncation
-      if (!is.null(self$truncation))
-        ld <- ld - self$tf_log_density_offset(tf_parameters)
-
-      ld
-    },
-
-    tf_fetch_parameters = function (dag) {
+    tf_fetch_parameters = function(dag) {
       # fetch the tensors corresponding to this node's parameters from the
       # environment, and return them in a named list
 
@@ -415,7 +474,7 @@ distribution_node <- R6Class (
 
     },
 
-    tf_log_density_offset = function (parameters) {
+    tf_log_density_offset = function(parameters) {
 
       # calculate the log-adjustment to the truncation term of the density
       # function i.e. the density of a distribution, truncated between a and b,
@@ -448,7 +507,7 @@ distribution_node <- R6Class (
 
     },
 
-    add_parameter = function (parameter, name) {
+    add_parameter = function(parameter, name) {
 
       parameter <- to_node(parameter)
       self$add_child(parameter)
@@ -456,21 +515,21 @@ distribution_node <- R6Class (
 
     },
 
-    tf_log_density_function = function (x, parameters) {
+    tf_log_density_function = function(x, parameters, dag) {
 
-      self$tf_distrib(parameters)$log_prob(x)
+      self$tf_distrib(parameters, dag)$log_prob(x)
 
     },
 
-    tf_cdf_function = function (x, parameters) {
+    tf_cdf_function = function(x, parameters) {
 
-      self$tf_distrib(parameters)$cdf(x)
+      self$tf_distrib(parameters, dag)$cdf(x)
 
     },
 
-    tf_log_cdf_function = function (x, parameters) {
+    tf_log_cdf_function = function(x, parameters) {
 
-      self$tf_distrib(parameters)$log_cdf(x)
+      self$tf_distrib(parameters, dag)$log_cdf(x)
 
     }
 
@@ -486,10 +545,13 @@ node_classes_module <- module(node,
 
 
 # shorthand for distribution parameter constructors
-distrib <- function (distribution, ...) {
+distrib <- function(distribution, ...) {
+
+  check_tf_version("error")
 
   # get and initialize the distribution, with a default value node
-  constructor <- get(paste0(distribution, '_distribution'))
+  constructor <- get(paste0(distribution, "_distribution"),
+                     envir = parent.frame())
   distrib <- constructor$new(...)
 
   # return the user-facing representation of the node as a greta array
@@ -499,13 +561,14 @@ distrib <- function (distribution, ...) {
 }
 
 # shorthand to speed up op definitions
-op <- function (...)
+op <- function(...) {
   as.greta_array(operation_node$new(...))
+}
 
 # helper function to create a variable node
 # by default, make x (the node
 # containing the value) a free parameter of the correct dimension
-vble <- function (truncation, dim = 1) {
+vble <- function(truncation, dim = 1) {
 
   if (is.null(truncation))
     truncation <- c(-Inf, Inf)
@@ -517,5 +580,5 @@ vble <- function (truncation, dim = 1) {
 }
 
 node_constructors_module <- module(distrib,
-                              op,
-                              vble)
+                                   op,
+                                   vble)
