@@ -7,12 +7,12 @@
 # 'iter' must be a length-two vector giving the total warmup and sampling
 #   iterations respectively
 # 'pb_update' gives the number of iterations between updates of the progress bar
-create_progress_bar <- function (phase, iter, pb_update, ...) {
+create_progress_bar <- function(phase, iter, pb_update, width, ...) {
 
   # name for formatting
   name <- switch(phase,
-                 warmup = '  warmup',
-                 sampling = 'sampling')
+                 warmup = "  warmup",
+                 sampling = "sampling")
 
   # total iterations for bat
   iter_this <- switch(phase,
@@ -22,7 +22,7 @@ create_progress_bar <- function (phase, iter, pb_update, ...) {
   # pad the frmat so that the width iterations counter is the same for both
   # warmup and sampling
   digit_diff <- nchar(max(iter)) - nchar(iter_this)
-  count_pad <- paste0(rep(' ', 2 * digit_diff), collapse = '')
+  count_pad <- paste0(rep(" ", 2 * digit_diff), collapse = "")
 
   # formatting
   format_text <- sprintf("  %s :bar %s:iter/:total | eta: :eta :rejection",
@@ -31,16 +31,21 @@ create_progress_bar <- function (phase, iter, pb_update, ...) {
 
   pb <- progress::progress_bar$new(format = format_text,
                                    total = iter_this,
-                                   incomplete = ' ',
+                                   width = width,
+                                   incomplete = " ",
                                    clear = FALSE,
+                                   current = "=",
                                    show_after = 0,
+                                   force = TRUE,
                                    ...)
 
   # add the increment information and return
   pb_update <- round(pb_update)
 
-  if (!is.numeric(pb_update) || length(pb_update) != 1 || !is.finite(pb_update) || pb_update <= 0)
-    stop ("pb_update must be a finite, positive, scalar integer")
+  if (!is.numeric(pb_update) || length(pb_update) != 1 ||
+      !is.finite(pb_update) || pb_update <= 0) {
+    stop("pb_update must be a finite, positive, scalar integer")
+  }
 
   assign("pb_update", pb_update, envir = pb$.__enclos_env__)
 
@@ -54,35 +59,39 @@ create_progress_bar <- function (phase, iter, pb_update, ...) {
 # 'pb' is a progress_bar R6 object created by create_progress_bar
 # 'it' is the current iteration
 # 'rejects' is the total number of rejections so far due to numerical instability
-iterate_progress_bar <- function (pb, it, rejects) {
+iterate_progress_bar <- function(pb, it, rejects, chains, file = NULL) {
 
   increment <- pb$.__enclos_env__$pb_update
 
   if (it %% increment == 0) {
 
     if (rejects > 0) {
-      reject_perc <- 100 * rejects / it
+      reject_perc <- 100 * rejects / (it * chains)
       if (reject_perc < 1) {
-        reject_perc_string <- '<1'
+        reject_perc_string <- "<1"
       } else {
         reject_perc_string <- prettyNum(round(reject_perc))
       }
       # pad the end of the line to keep the update bar a consistent width
       pad_char <- pmax(0, 2 - nchar(reject_perc_string))
-      pad <- paste0(rep(' ', pad_char), collapse = '')
+      pad <- paste0(rep(" ", pad_char), collapse = "")
 
-      reject_text <- paste0('| ', reject_perc_string, '% bad', pad)
+      reject_text <- paste0("| ", reject_perc_string, "% bad", pad)
     } else {
-      reject_text <- '         '
+      reject_text <- "         "
     }
 
     total <- pb$.__enclos_env__$private$total
     iter_pretty <- prettyNum(it, width = nchar(total))
 
     amount <- ifelse(it > 0, increment, 0)
-    invisible(pb$tick(amount,
-                      tokens = list(iter = iter_pretty,
-                                    rejection = reject_text)))
+
+    # tick the progess bar and record the output message
+    # (or print it if file = NULL)
+    record(pb$tick(amount,
+                   tokens = list(iter = iter_pretty,
+                                 rejection = reject_text)),
+           file = file)
 
   }
 
