@@ -40,10 +40,12 @@
 #' max <- min ^ 2
 #' d <- min + variable(0, 1, dim = nrow(iris)) * (max - min)
 #' }
-variable <- function (lower = -Inf, upper = Inf, dim = 1) {
+variable <- function(lower = -Inf, upper = Inf, dim = 1) {
+
+  check_tf_version("error")
 
   if (inherits(lower, "greta_array") | inherits(upper, "greta_array"))
-    stop ('lower and upper must be fixed, they cannot be another greta array')
+    stop("lower and upper must be fixed, they cannot be another greta array")
 
   node <- variable_node$new(lower, upper, dim)
   as.greta_array(node)
