@@ -1,9 +1,9 @@
-context('operators')
+context("operators")
 
-test_that('arithmetic operators work as expected', {
+test_that("arithmetic operators work as expected", {
 
   skip_if_not(check_tf_version())
-  source('helpers.R')
+  source("helpers.R")
 
   a <- randn(25, 4)
   b <- randn(25, 4)
@@ -20,10 +20,10 @@ test_that('arithmetic operators work as expected', {
 
 })
 
-test_that('logical operators work as expected', {
+test_that("logical operators work as expected", {
 
   skip_if_not(check_tf_version())
-  source('helpers.R')
+  source("helpers.R")
 
   a <- randn(25, 4) > 0
   b <- randn(25, 4) > 0
@@ -36,10 +36,10 @@ test_that('logical operators work as expected', {
 
 })
 
-test_that('relational operators work as expected', {
+test_that("relational operators work as expected", {
 
   skip_if_not(check_tf_version())
-  source('helpers.R')
+  source("helpers.R")
 
   a <- randn(25, 4)
   b <- randn(25, 4)
@@ -53,10 +53,10 @@ test_that('relational operators work as expected', {
 
 })
 
-test_that('random strings of operators work as expected', {
+test_that("random strings of operators work as expected", {
 
   skip_if_not(check_tf_version())
-  source('helpers.R')
+  source("helpers.R")
 
   for (i in 1:10) {
 
@@ -65,9 +65,9 @@ test_that('random strings of operators work as expected', {
 
     # generate a 5-deep random function of operations
     fun <- gen_opfun(5,
-                     ops = c('+', '-', '*',
-                             '/', '%/%', '&',
-                             '|', '<', '>'))
+                     ops = c("+", "-", "*",
+                             "/", "&",
+                             "|", "<", ">"))
 
     r_out <- fun(a, b)
     greta_out <- grab(fun(as_data(a), as_data(b)))
@@ -87,13 +87,19 @@ test_that('random strings of operators work as expected', {
 
 })
 
-test_that('%*% errors informatively', {
+test_that("%*% errors informatively", {
 
   skip_if_not(check_tf_version())
+  source("helpers.R")
+
   a <- ones(3, 4)
   b <- ones(1, 4)
+  c <- ones(2, 2, 2)
 
   expect_error(a %*% b,
-               'incompatible dimensions: 3x4 vs 1x4')
+               "incompatible dimensions: 3x4 vs 1x4")
+
+  expect_error(a %*% c,
+               "only two-dimensional greta arrays can be matrix-multiplied")
 
 })
